use owl_macro::model;
use serde::{Deserialize, Serialize};

pub use crate as owl;

use crate::db::id::Id;

/// Represents a geographical location with an ID, coordinates, and optional address
#[derive(Debug, PartialEq)]
#[model]
pub struct Location {
    pub id: Id,
    /// A common name for the `Location`
    pub location_name: Option<String>,
    /// Geographical coordinates of the location
    pub geo: (f64, f64),
    /// The street address
    pub location_address: Option<String>,
    /// The city name
    pub city: Option<String>,
    /// The state or province name
    pub location_state: Option<String>,
    /// The postal or ZIP code
    pub postal_code: Option<String>,
    /// The country name
    pub country: Option<String>,
}

/// Represents geographical coordinates with latitude and longitude
#[derive(Debug, Clone, Serialize, Deserialize, PartialEq)]
pub struct GeoCoordinates {
    /// The latitude coordinate
    pub latitude: f64,
    /// The longitude coordinate
    pub longitude: f64,
}
