use owl_macro::model;
use serde::{Deserialize, Serialize};

pub use crate as owl;

use crate::{
    db::{
        field::{Historic, Historicable},
        id::Id,
        query_t,
        relation::IdRef,
        Database, Model,
    },
    Identifiable,
};

use super::{location::Location, vehicle::Vehicle};

#[derive(Debug, PartialEq)]
#[model]
pub struct Person {
    pub id: Id,
    /// The person's first name
    pub first_name: Historic<String>,
    /// The person's last name
    pub last_name: Historic<String>,
    /// The person's date of birth.
    pub date_of_birth: Option<chrono::NaiveDateTime>,
    /// The person's gender.
    pub gender: Option<Gender>,
    /// Associated locations
    pub locations: Historic<Vec<IdRef<Location>>>,
}

#[derive(Deserialize, Serialize, Debug, PartialEq, Clone)]
pub enum Gender {
    Male,
    Female,
    Other,
}

impl Person {
    pub fn new(first_name: &str, last_name: &str) -> Self {
        Self {
            id: Id::new_ulid(),
            first_name: first_name.to_string().historic(),
            last_name: last_name.to_string().historic(),
            date_of_birth: None,
            gender: None,
            locations: Historic::new(),
        }
    }

    pub fn new_id(id: &str, first_name: &str, last_name: &str) -> Self {
        Self {
            id: Id::String(id.to_string()),
            first_name: first_name.to_string().historic(),
            last_name: last_name.to_string().historic(),
            date_of_birth: None,
            gender: None,
            locations: Historic::new(),
        }
    }

    pub fn vehicles(&self, db: &Database) -> Vec<Model<Vehicle>> {
        query_t(
            |vehicle: &Vehicle| {
                if let Some(owner) = &vehicle.owned_by {
                    if *owner == self.reference() {
                        return true;
                    }
                }

                false
            },
            db,
        )
    }

    /*
        pub async fn remove_email(&self, email: uuid::Uuid) {
        Relation::remove("relate_person_email", &self.id, &email).await
    }pub async fn add_email(&self, email: uuid::Uuid) {
        Relation::add("relate_person_email", &self.id, &email).await
    }

    pub async fn emails(&self) -> Vec<MailAddress> {
        Relation::find_relations("relate_person_email", "mail_address", &self.id).await
    }
    pub async fn phone_numbers(&self) -> Vec<PhoneNumber> {
        Relation::find_relations("relate_person_phone_number", "phone_number", &self.id).await
    }
         */
}
